<?php

/*
 * This file is part of the Comptoir-du-Libre software.
 * <https://gitlab.adullact.net/Comptoir/comptoir-du-libre>
 *
 * Copyright (c) ADULLACT   <https://adullact.org>
 *               Association des Développeurs et Utilisateurs de Logiciels Libres
 *               pour les Administrations et les Collectivités Territoriales
 *
 * Comptoir-du-Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public Organization as published
 * by the Free Software Foundation, either version 3 of the Organization, or
 * (at your option) any later version.
 *
 * You should have received a copy of the GNU Affero General Public Organization
 * along with this software. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 */

declare(strict_types=1);

namespace App\Tests\Functional\Organization;

use App\DataFixtures\AppOrganizationFixtures;
use App\Tests\Functional\TestHelperBreadcrumbTrait;
use App\Tests\Functional\TestHelperTrait;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Symfony\Component\HttpFoundation\Response;

/**
 * @group allow_mutation_testing_by_infection
 * @group gogogo
 */
class FunctionalTestOrganizationPagesTest extends WebTestCase
{
    use TestHelperTrait;
    use TestHelperBreadcrumbTrait;


    public function testAllOrganizationsPageAllowToChangeLocaleInUrl(): void
    {
        $client = static::createClient();
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/en/organisations/",
            expectedNewUrl: "/en/organizations/",
            expectedCurrentRoute: 'app_anonymous_organization_display_all_allow_to_change_locale_in_url',
            expectedNewRoute: 'app_anonymous_organization_display_all_organization',
        );
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/fr/organizations/",
            expectedNewUrl: "/fr/organisations/",
            expectedCurrentRoute: 'app_anonymous_organization_display_all_allow_to_change_locale_in_url',
            expectedNewRoute: 'app_anonymous_organization_display_all_organization',
        );
    }

    public function testOneOrganizationPageAllowToTruncateUrlToRedirectToAllOrganizationsPage(): void
    {
        $client = static::createClient();
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/en/organization/",
            expectedNewUrl: "/en/organizations/",
            expectedCurrentRoute: 'app_anonymous_organization_display_all_fix_url_typo',
            expectedNewRoute: 'app_anonymous_organization_display_all_organization',
        );
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/fr/organisation/",
            expectedNewUrl: "/fr/organisations/",
            expectedCurrentRoute: 'app_anonymous_organization_display_all_fix_url_typo',
            expectedNewRoute: 'app_anonymous_organization_display_all_organization',
        );
    }


    public function testOneOrganizationPageAllowToChangeLocaleInUrl(): void
    {
        $organizationId = AppOrganizationFixtures::ORGANIZATIONS[3]['id'];
        $organizationSlug = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['slug'];
        $client = static::createClient();
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/en/organisation/$organizationId/$organizationSlug/",
            expectedNewUrl: "/en/organization/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_allow_to_change_locale_in_url',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/fr/organization/$organizationId/$organizationSlug/",
            expectedNewUrl: "/fr/organisation/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_allow_to_change_locale_in_url',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
    }


    public function testOneOrganizationPageWithoutSlugRedirectToValidUrl(): void
    {
        $organizationId = AppOrganizationFixtures::ORGANIZATIONS[3]['id'];
        $organizationSlug = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['slug'];
        $client = static::createClient();
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/en/organization/$organizationId/",
            expectedNewUrl: "/en/organization/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_organization_without_slug',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/fr/organisation/$organizationId/",
            expectedNewUrl: "/fr/organisation/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_organization_without_slug',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
    }

    public function testOneOrganizationPageWithBadSlugRedirectToValidUrl(): void
    {
        $organizationId = AppOrganizationFixtures::ORGANIZATIONS[3]['id'];
        $organizationSlug = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['slug'];
        $client = static::createClient();
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/en/organization/$organizationId/not-valid-slug/",
            expectedNewUrl: "/en/organization/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_organization',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
        $this->checkRedirectionToNewUrl(
            client: $client,
            currentUrl: "/fr/organisation/$organizationId/not-valid-slug/",
            expectedNewUrl: "/fr/organisation/$organizationId/$organizationSlug/",
            expectedCurrentRoute: 'app_anonymous_organization_display_one_organization',
            expectedNewRoute: 'app_anonymous_organization_display_one_organization',
        );
    }

    public function testOneOrganizationPageDisplay(): void
    {
        $locale = 'en';
        $urlOrganizationPrefixSingle = 'organization';
        $urlOrganizationPrefixPlural = 'organizations';
        $organizationId = AppOrganizationFixtures::ORGANIZATIONS[3]['id'];
        $organizationSlug = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['slug'];
        $organizationName = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['name'];
        $organizationWebsite = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['website'];
        $organizationType = AppOrganizationFixtures::ORGANIZATIONS[$organizationId]['type'];

        $organizationUrl = "/$locale/$urlOrganizationPrefixSingle/$organizationId/$organizationSlug/";

        $client = static::createClient();
        $crawler = $client->request('GET', "$organizationUrl");
        $this->assertRouteSame('app_anonymous_organization_display_one_organization');
        $this->assertResponseStatusCodeSame(Response::HTTP_OK); // HTTP status code = 200

        // HTML content checks breadcrumb
        $breadcrumbLinks = [
            "/$locale/$urlOrganizationPrefixPlural/public-sector/" => "Public sector",
            "$organizationUrl" => "$organizationName",
        ];
        $this->checkHasValidBreadcrumb($crawler, $breadcrumbLinks, "$locale");

        $this->markTestIncomplete();
    }


}
